-- ****************************************************************************
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-D33
-- * (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- *		
-- * US Government Users Restricted Rights - Use, duplication or
-- * disclosure restricted by GSA ADP Schedule Contract with
-- * IBM Corp.
-- *
-- ****************************************************************************

-- ETT1: Extract data from Tivoli License Management database to the 
-- Tivoli Enterprise Data Warehouse

---------------------------------------------------
--drop/Create staging tables and other temp objects
---------------------------------------------------

--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.stage_RUN_DATE;

--#EXECUTE_AT_TARGET
--Create table used to timestamp start of ETL1. This will determine
-- timestamp on measurements inserted into warehouse.
create table cod.stage_run_date (
    run_time                 timestamp
);

--#EXECUTE_AT_TARGET
insert into cod.stage_run_date values (current timestamp - current timezone);
 
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TRIGGER COD.MAP_OS;

--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_COMP;

--#EXECUTE_AT_TARGET
create table cod.stage_comp (
    cust_name           varchar(120)    not null,
    hostname            varchar(60)     not null,
    os_name             varchar(10)     not null,
    os_version          varchar(60)     not null,
    prod_name           varchar(240)    not null,
    prod_version        varchar(64)     not null,
    division_name       varchar(120)    not null
);

--#EXECUTE_AT_TARGET
create index cod.stgcmpI1 on cod.stage_comp (
    division_name
);

--#EXECUTE_AT_TARGET
create index cod.stgcmpI2 on cod.stage_comp (
    hostname
);

--#EXECUTE_AT_TARGET
create index cod.stgcmpI3 on cod.stage_comp (
    os_version
);

--#EXECUTE_AT_TARGET
create index cod.stgcmpI4 on cod.stage_comp (
    cust_name
);

--#EXECUTE_AT_TARGET
create index cod.stgcmpI5 on cod.stage_comp (
    prod_name
);

--#EXECUTE_AT_TARGET
create index cod.stgcmpI6 on cod.stage_comp (
    prod_version
);

--#EXECUTE_AT_TARGET
create index cod.stgcmpI7 on cod.stage_comp (
    os_name
);

--#EXECUTE_AT_TARGET
--#OVERRIDE_TERMINATOR $
create trigger cod.map_os
no cascade before insert on cod.stage_comp
referencing new as n
for each row mode db2sql
begin atomic
    if  n.os_version like 'Windows  NT 4.0%' THEN
        set n.os_version='Windows NT';
    elseif  n.os_version like 'Windows  NT 5.0%' THEN
        set n.os_version='Windows 2000';
    elseif  n.os_version like 'Windows  95%' THEN
        set n.os_version='Win95';
    elseif  n.os_version like 'Windows  98%' THEN
        set n.os_version='Win98';
    elseif  n.os_version like 'Linux%' THEN
        set n.os_version='Linux';
    elseif  n.os_version like 'AIX%' THEN
        set n.os_version='AIX';
    elseif  n.os_version like 'SunOS%' THEN
        set n.os_version='SunOS';
    end if;
end $ 

--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TRIGGER COD.CORR_ID_MEASURE;

--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_MEASUREMENT;

--#EXECUTE_AT_TARGET
create table cod.stage_measurement (
    corr_id             INTEGER         not null,
    cust_name           varchar(120)    not null,
    division_name       varchar(120)    not null,
    os_version          varchar(60)     not null,
    prod_name           varchar(254)    not null,     
    count_measure       INTEGER         not null
);

--#EXECUTE_AT_TARGET
create index cod.stgmesI1 on cod.stage_measurement (
    corr_id
);

--#EXECUTE_AT_TARGET
create index cod.stgmesI2 on cod.stage_measurement (
    Prod_Name
);

--#EXECUTE_AT_TARGET
create index cod.stgmesI3 on cod.stage_measurement (
    Division_Name
);

--#EXECUTE_AT_TARGET
create index cod.stgmesI4 on cod.stage_measurement (
    Os_Version
);

--#EXECUTE_AT_TARGET
create index cod.stgmesI5 on cod.stage_measurement (
    Cust_Name
);

--#OVERRIDE_TERMINATOR $
--#EXECUTE_AT_TARGET
CREATE TRIGGER COD.CORR_ID_MEASURE NO CASCADE BEFORE 
INSERT ON COD.STAGE_MEASUREMENT
REFERENCING  NEW AS newrows
FOR EACH ROW
MODE DB2SQL set corr_id = nextval for cod.meas_coor_id_seq$

--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_NEW_COMP;

--#EXECUTE_AT_TARGET
create table cod.stage_new_comp (
    cust_name           varchar(120)    not null,
    hostname            varchar(60)     not null,
    os_name             varchar(10)     not null,
    os_version          varchar(60)     not null,
    prod_name           varchar(254)    not null,
    division_name       varchar(120)    not null
);

--#EXECUTE_AT_TARGET
create index cod.stgncpI1 on cod.stage_new_comp (
    os_version
);

--#EXECUTE_AT_TARGET
create index cod.stgncpI2 on cod.stage_new_comp (
    prod_name
);

--#EXECUTE_AT_TARGET
create index cod.stgncpI3 on cod.stage_new_comp (
    cust_name
);

--#EXECUTE_AT_TARGET
create index cod.stgncpI4 on cod.stage_new_comp (
    division_name
);

--#EXECUTE_AT_TARGET
create index cod.stgncpI5 on cod.stage_new_comp (
    hostname
);

--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_IP_HOST;

--#EXECUTE_AT_TARGET
create table cod.stage_ip_host (
    name               varchar(60)
);

--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_PROD;

--#EXECUTE_AT_TARGET
create table cod.stage_prod (
    name               varchar(254)      not null
);   

--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TRIGGER COD.CORR_ID_HOST_OS;

--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_HOST_OS;

--#EXECUTE_AT_TARGET
create table cod.stage_host_os (
    corr_id             INTEGER           not null,
    host_name           varchar(60)       not null,
    os_name             varchar(40)       not null
);      

--#EXECUTE_AT_TARGET
create index cod.stage_host_osI1 on cod.stage_host_os
(
    host_name,
    corr_id
);      


--#OVERRIDE_TERMINATOR $
--#EXECUTE_AT_TARGET
CREATE TRIGGER COD.CORR_ID_HOST_OS NO CASCADE BEFORE 
INSERT ON COD.STAGE_HOST_OS
REFERENCING  NEW AS newrows
FOR EACH ROW
MODE DB2SQL set corr_id = nextval for cod.meas_coor_id_seq$

-----------------------------------------------------------------
--Delete any data that may be garbage data left over
-- from a previous failed run
-- NOTE: since the Tivoli License Manager measurements are generated daily 
--       based on counts taken on the source database, if an ETL1 run fails
--       and is not re-run that same day, the measurements for that
--       day can never be recovered.
-------------------------------------------------------------------

--Any division data with missing attributes is garbage data
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP;

--#EXECUTE_AT_TARGET
create table cod.stage_temp (
    Comp_ID        INTEGER not null
     );

--#EXECUTE_AT_TARGET
create index cod.stage_tempI on cod.stage_temp
(
    Comp_ID
);

--#EXECUTE_AT_TARGET
insert into cod.stage_temp (Comp_ID)
   select distinct custAttr.Comp_ID
       from twg.compAttr prodAttr, twg.CompAttr osAttr, twg.CompAttr custAttr
       where prodAttr.AttrTyp_Cd = 'COD_PRODUCT'
         and osAttr.AttrTyp_Cd = 'COD_OS'
         and custAttr.AttrTyp_Cd = 'COD_CUSTOMER'
         and prodAttr.Comp_ID = custAttr.Comp_ID
         and osAttr.Comp_ID = custAttr.Comp_ID;

--#EXECUTE_AT_TARGET
delete from twg.comp divComp
where CompTyp_cd = 'COD_DIVISION'
  and not exists 
      (select 1
            from cod.stage_temp temp
            where temp.Comp_ID = divComp.Comp_ID);

--Only one set of measurements can be taken per 24-hour day. Any existing
-- measurements with today's date must be from a (failed) run earlier today 
-- which is being re-done
--#EXECUTE_AT_TARGET
delete from twg.Msmt msmt
where msmt.MsmtTyp_ID = (select MsmtTyp_ID from twg.msmtTyp msmttyp 
                         where msmttyp.Msrc_Cd = 'COD'
                           and msmttyp.MsmtTyp_Nm = 'Number of Installations')
  and Msmt_Strt_Dt = (select DATE(run_time) from cod.stage_run_date);

--Any COD_OS components not contained (LCONT) by some host is garbage data.
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP;

--#EXECUTE_AT_TARGET
create table cod.stage_temp (
    Comp_ID        INTEGER not null
     );

--#EXECUTE_AT_TARGET
create index cod.stage_tempI on cod.stage_temp
(
    Comp_ID
);

--#EXECUTE_AT_TARGET
insert into cod.stage_temp
    select distinct reln.Comp_Target_ID
    from twg.CompReln reln
    where reln.RelnTyp_Cd = 'LCONT';
    
--#EXECUTE_AT_TARGET
delete from twg.Comp osComp
where CompTyp_Cd = 'COD_OS'
  and not exists 
      (select 1 from cod.stage_temp temp 
       where temp.Comp_ID = osComp.Comp_ID);
   
------------------------------------------------------
-- Collect data from source database. Populate base 
--  staging table
------------------------------------------------------

--#INSERT_INTO_TARGET
insert into cod.stage_comp
  select cust.name, agt.hostname, agt.os_name, agt.os_version, comp.name, 
         comp.version, div.name 
  from adm.tedw_product comp, adm.tedw_agent agt, adm.tedw_install agt_inv, 
       adm.tedw_division div, adm.tedw_customer cust
  where agt_inv.end_time > CURRENT TIMESTAMP - CURRENT TIMEZONE and
        agt_inv.product_id = comp.id and
        agt_inv.agent_id = agt.id and
        agt.division_id = div.id and
        agt.customer_id = cust.id and
        ucase(comp.name) not like 'IBM TIVOLI LICENSE MANAGER AGENT%'
UNION select cust.name, agt.hostname, agt.os_name, agt.os_version, 
             'IBM Tivoli License Manager Agent', agt.version, div.name
  from adm.tedw_agent agt, adm.tedw_division div, adm.tedw_customer cust
  where agt.division_id = div.id and
        agt.customer_id = cust.id;


--Synthesize measurements based on base staging table 
--#EXECUTE_AT_TARGET
insert into cod.stage_measurement
  select 1, cust_name, division_name, os_version, 
         concat(concat(prod_name,' '),prod_version), count(prod_name)
  from cod.stage_comp
  group by cust_name, division_name, os_version, 
           concat(concat(prod_name,' '),prod_version);


--select non-duplicate table entries
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP1;

--#EXECUTE_AT_TARGET
create table cod.stage_temp1
(
    Os_Comp_Id      INTEGER not null,
    Hostname        varchar(60) not null,
    Os_Version      varchar(60) not null
);

--#EXECUTE_AT_TARGET
create index cod.stage_temp1I on cod.stage_temp1
(
    Os_Comp_Id
);

--#EXECUTE_AT_TARGET
insert into cod.stage_temp1 (Os_Comp_Id, Hostname, Os_Version)
   select distinct osComp.Comp_ID, hostComp.Comp_Nm, osComp.Comp_Nm
        from  twg.Comp osComp, twg.comp hostComp,
              twg.CompReln lcontReln
        where (hostComp.CompTyp_Cd = 'IP_HOST'
               or hostComp.CompTyp_cd = 'COD_HOST')
          and hostComp.Comp_ID = lcontReln.Comp_Source_ID
          and lcontReln.RelnTyp_CD = 'LCONT'
          and lcontReln.Comp_Target_ID = osComp.Comp_ID 
          and osComp.CompTyp_Cd = 'COD_OS';


--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP2;

--#EXECUTE_AT_TARGET
create table cod.stage_temp2
(
    Prod_Comp_Id    INTEGER not null,
    Hostname        varchar(60) not null,
    Os_Version      varchar(60) not null,
    Prod_Name       varchar(254) not null
);

--#EXECUTE_AT_TARGET
create index cod.stgtp2I1 on cod.stage_temp2
(
    prod_name
);

--#EXECUTE_AT_TARGET
create index cod.stgtp2I2 on cod.stage_temp2
(
    prod_comp_id
);

--#EXECUTE_AT_TARGET
create index cod.stgtp2I3 on cod.stage_temp2
(
    os_version
);
                   
--#EXECUTE_AT_TARGET
insert into cod.stage_temp2 (Prod_Comp_Id, Hostname, Os_Version, Prod_Name)
   select distinct prodComp.Comp_ID, temp.Hostname, temp.Os_Version, prodComp.Comp_Nm
        from  twg.Comp prodComp, twg.CompReln instonReln,
              cod.stage_temp1 temp
        where prodComp.CompTyp_Cd = 'COD_PRODUCT'
          and instonReln.RelnTyp_CD = 'INSTON'
          and instonReln.Comp_Target_ID = temp.Os_Comp_ID
          and instonReln.Comp_Source_ID = prodComp.Comp_ID;


--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP3;

--#EXECUTE_AT_TARGET
create table cod.stage_temp3
(
    Hostname        varchar(60)  not null,
    Os_Version      varchar(60)  not null,
    Prod_Name       varchar(254) not null,
    Division_Name   varchar(120) not null,
    Cust_Name       varchar(120) not null
);

--#EXECUTE_AT_TARGET
create index cod.stgtp3I1 on cod.stage_temp3
(
    Hostname
);

--#EXECUTE_AT_TARGET
create index cod.stgtp3I2 on cod.stage_temp3
(
    Os_Version
);

--#EXECUTE_AT_TARGET
create index cod.stgtp3I3 on cod.stage_temp3
(
    Prod_Name
);

--#EXECUTE_AT_TARGET
create index cod.stgtp3I4 on cod.stage_temp3
(
    Division_Name
);

--#EXECUTE_AT_TARGET
create index cod.stgtp3I5 on cod.stage_temp3
(
    Cust_Name
);

--#EXECUTE_AT_TARGET
insert into cod.stage_temp3 (Hostname, Os_Version, Prod_Name, Division_Name, Cust_Name) 
select distinct temp.Hostname, temp.Os_Version, temp.Prod_Name,
                divComp.Comp_Nm, custAttr.CompAttr_Val 
from twg.Comp Divcomp, twg.CompReln pchildReln, twg.CompAttr custAttr, 
     cod.stage_comp stage, cod.stage_temp2 temp 
where divComp.CompTyp_Cd = 'COD_DIVISION' 
  and pchildReln.RelnTyp_CD = 'PCHILD' 
  and pchildReln.Comp_Source_ID = temp.Prod_Comp_ID 
  and pchildReln.Comp_Target_ID = divComp.Comp_ID
  and custAttr.AttrTyp_Cd = 'COD_CUSTOMER' 
  and divComp.Comp_ID = custAttr.Comp_ID
  and divComp.Comp_Nm = stage.division_name 
  and temp.Hostname = stage.hostname 
  and temp.Os_Version = stage.os_version 
  and temp.Prod_Name = concat(concat(stage.prod_name,' '),stage.prod_version)
  and custAttr.CompAttr_Val = stage.cust_name;


--#EXECUTE_AT_TARGET
insert into cod.stage_new_comp 
select distinct cust_name, hostname, os_name, os_version, 
       concat(concat(prod_name,' '),prod_version), division_name
from cod.stage_comp stage
where not exists (select 1 from  cod.stage_temp3 temp
                  where temp.hostname = stage.hostname
                   and  temp.os_version = stage.os_version
                   and  temp.prod_name = concat(concat(stage.prod_name,' '),
                                                  stage.prod_version)
                   and  temp.division_name = stage.division_name
                   and  temp.cust_name = stage.cust_name);

--Insert division components. Only insert for new combinations
-- of division/product/os/customer
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP1;

--#EXECUTE_AT_TARGET
create table cod.stage_temp1 (
    Comp_ID         INTEGER not null,
    Os_Version      varchar(60)  not null,
    Prod_Name       varchar(254) not null,
    Cust_Name       varchar(120) not null
);

--#EXECUTE_AT_TARGET
create index cod.stage_temp1I on cod.stage_temp1
(
    Comp_ID
);

--#EXECUTE_AT_TARGET
insert into cod.stage_temp1 (Comp_ID, Os_Version, Prod_Name, Cust_Name)
   select distinct custAttr.Comp_ID, osAttr.compAttr_Val,
             prodAttr.compAttr_Val, custAttr.compAttr_Val
       from twg.compAttr prodAttr, twg.CompAttr osAttr, twg.CompAttr custAttr
       where prodAttr.AttrTyp_Cd = 'COD_PRODUCT'
         and osAttr.AttrTyp_Cd = 'COD_OS'
         and custAttr.AttrTyp_Cd = 'COD_CUSTOMER'
         and prodAttr.Comp_ID = custAttr.Comp_ID
         and osAttr.Comp_ID = custAttr.Comp_ID;
         
                 
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP2;

--#EXECUTE_AT_TARGET
create table cod.stage_temp2 (
    Os_Version      varchar(60)  not null,
    Prod_Name       varchar(254) not null,
    Division_Name   varchar(120) not null,
    Cust_Name       varchar(120) not null
);

--#EXECUTE_AT_TARGET
create index cod.stgp2I1 on cod.stage_temp2
(
    Os_Version
);

--#EXECUTE_AT_TARGET
create index cod.stgp2I2 on cod.stage_temp2
(
    Prod_Name
);

--#EXECUTE_AT_TARGET
create index cod.stgp2I3 on cod.stage_temp2
(
    Division_Name
);

--#EXECUTE_AT_TARGET
create index cod.stgp2I4 on cod.stage_temp2
(
    Cust_Name
);

--#EXECUTE_AT_TARGET
insert into cod.stage_temp2 (Os_Version, Prod_Name,
                            Division_Name, Cust_Name)
select temp.Os_Version, temp.Prod_Name,
       comp.comp_nm, temp.Cust_Name
from twg.comp comp, cod.stage_temp1 temp 
where comp.compTyp_Cd = 'COD_DIVISION'
   and comp.Comp_ID = temp.Comp_ID;
          
--#EXECUTE_AT_TARGET
insert into twg.Comp (comp_id, CompTyp_Cd, Centr_Cd, Cust_ID,
                      Comp_Corr_id, Comp_Nm, Comp_Strt_DtTm, Comp_End_DtTm)
       select 1, 'COD_DIVISION', 'CDW', 1, stage.corr_id, 
              stage.division_name, CURRENT TIMESTAMP - CURRENT TIMEZONE, 
              '9999-01-01-12.00.00.000000'
       from cod.stage_measurement stage
       where not exists (select 1
        from cod.stage_temp2 temp 
        where temp.os_version = stage.os_version
          and temp.prod_name = stage.prod_name
          and temp.division_name = stage.division_name
          and temp.cust_name = stage.cust_name);
 
--Update attributes for NEWLY added cod_division components
--use Comp_Corr_ID to relate component instance to attributes
--#EXECUTE_AT_TARGET
insert into twg.CompAttr (compAttr_ID, Comp_ID, AttrTyp_Cd, CompAttr_Strt_Dttm,
                        CompAttr_End_Dttm, CompAttr_val)
       select 1, Comp.Comp_ID, 'COD_PRODUCT', comp.comp_Strt_Dttm,
              Comp.Comp_End_Dttm, stage.prod_name
       from twg.Comp comp,  cod.stage_measurement stage
       where comp.CompTyp_Cd = 'COD_DIVISION'
         and  comp.Comp_Corr_ID = stage.corr_id
       UNION
       select 1, Comp.Comp_ID, 'COD_CUSTOMER', comp.comp_Strt_Dttm,
              Comp.Comp_End_Dttm, stage.cust_name
       from twg.Comp comp,  cod.stage_measurement stage
       where comp.CompTyp_Cd = 'COD_DIVISION'
         and  comp.Comp_Corr_ID = stage.corr_id
       UNION
       select 1, Comp.Comp_ID, 'COD_OS', comp.comp_Strt_Dttm,
              Comp.Comp_End_Dttm, stage.os_version
       from twg.Comp comp,  cod.stage_measurement stage
       where comp.CompTyp_Cd = 'COD_DIVISION'
         and  comp.Comp_Corr_ID = stage.corr_id;

--The measurement date and time are current date and time
--since the measurements are simply a snapshot of the database
--taken at ETL1 time
--#EXECUTE_AT_TARGET
insert into TWG.Msmt (Msmt_ID, Comp_id, MsmtTyp_ID, TmSum_cd, Msmt_Min_Val, 
                  Msmt_Max_Val, Msmt_Strt_Dt, Msmt_strt_Tm)
select 1, Comp.Comp_ID, MsmtTyp.MsmtTyp_ID, 'D', stage.count_measure,
       stage.count_measure, DATE(run_date.run_time),
       TIME('00:00:00')
from twg.Comp comp, twg.MsmtTyp MsmtTyp, cod.stage_measurement stage,
     twg.CompAttr prodAttr, twg.CompAttr custAttr, twg.CompAttr osAttr,
     cod.stage_run_date run_date
where MsmtTyp.MUnit_Cd = 'QTY' 
  and MsmtTyp.MSrc_cd = 'COD'
  and Comp.CompTyp_Cd = 'COD_DIVISION'
  and Comp.Comp_Nm = stage.division_name
  and Comp.Comp_ID = prodAttr.Comp_ID
  and prodAttr.AttrTyp_Cd = 'COD_PRODUCT'
  and prodAttr.CompAttr_Val = stage.prod_name
  and Comp.Comp_ID = custAttr.Comp_ID
  and custAttr.AttrTyp_Cd = 'COD_CUSTOMER'
  and custAttr.CompAttr_Val = stage.cust_name
  and Comp.Comp_ID = osAttr.Comp_ID
  and osAttr.AttrTyp_Cd = 'COD_OS'
  and osAttr.CompAttr_Val = stage.os_version;

--Stage new product components
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP;

--#EXECUTE_AT_TARGET
create table cod.stage_temp (
    Prod_Name       varchar(254) not null
);

--#EXECUTE_AT_TARGET
create index cod.stage_tempI on cod.stage_temp
(
    Prod_Name
);

--#EXECUTE_AT_TARGET
insert into cod.stage_temp
  select distinct comp.Comp_Nm
  from twg.Comp comp
  where comp.CompTyp_Cd = 'COD_PRODUCT';
  
--#EXECUTE_AT_TARGET
insert into cod.stage_prod 
  select distinct meas.prod_name 
  from cod.stage_measurement meas
  where not exists 
  (select 1 from cod.stage_temp temp
   where temp.prod_name = meas.prod_name);

--Insert new product components discovered on this ETL run
--#EXECUTE_AT_TARGET
insert into twg.Comp (Comp_id, CompTyp_Cd, Centr_Cd, Cust_ID,
                      Comp_Nm, Comp_Strt_DtTm, Comp_End_DtTm)
       select 1, 'COD_PRODUCT', 'CDW', 1, 
              stage.name, CURRENT TIMESTAMP - CURRENT TIMEZONE, 
              '9999-01-01-12.00.00.000000'
       from cod.stage_prod stage;

--Update PCHILD relationships between newly inserted division components
-- new (or existing) product components that they relate to. 
-- The division to product
-- relationship is many to many.
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP;

--#EXECUTE_AT_TARGET
create table cod.stage_temp (
    Division_Name   varchar(120) not null,
    Prod_Name       varchar(254) not null,
    Os_Version      varchar(60)  not null,
    Cust_Name       varchar(120) not null
);

--#EXECUTE_AT_TARGET
create index cod.stgtmpI1 on cod.stage_temp
(   
    Division_Name
);

--#EXECUTE_AT_TARGET
create index cod.stgtmpI2 on cod.stage_temp
(   
    Prod_Name
);

--#EXECUTE_AT_TARGET
create index cod.stgtmpI3 on cod.stage_temp
(   
    Os_Version
);

--#EXECUTE_AT_TARGET
create index cod.stgtmpI4 on cod.stage_temp
(   
    Cust_Name
);

--#EXECUTE_AT_TARGET
insert into cod.stage_temp (Division_Name, Prod_Name, Os_Version, Cust_Name)
        select distinct divComp.Comp_nm, prodComp.Comp_nm, osAttr.compAttr_Val, custAttr.CompAttr_Val
        from twg.CompReln reln, twg.Comp divComp, twg.Comp prodComp,
                       twg.CompAttr osAttr, twg.CompAttr prodAttr,
                       twg.compAttr custAttr 
            where reln.RelnTyp_Cd = 'PCHILD'
              and prodComp.CompTyp_cd = 'COD_PRODUCT'
              and divComp.CompTyp_cd = 'COD_DIVISION'
              and osAttr.attrTyp_cd = 'COD_OS'
              and prodAttr.attrTyp_cd = 'COD_PRODUCT'
              and custAttr.attrTyp_cd = 'COD_CUSTOMER'
              and prodComp.Comp_id = reln.Comp_source_ID
              and divComp.Comp_id = reln.Comp_Target_ID
              and osAttr.Comp_id = divComp.Comp_id
              and prodAttr.Comp_id = divComp.Comp_id
              and prodAttr.CompAttr_Val = prodComp.Comp_nm
              and custAttr.Comp_id = divComp.comp_id;

--#EXECUTE_AT_TARGET
insert into twg.CompReln (CompReln_ID, Comp_Source_ID, Comp_Target_ID,
                          RelnTyp_Cd, CompReln_Strt_Dttm, CompReln_End_Dttm) 
  select distinct 1, prodComp.Comp_ID, divComp.Comp_ID, 'PCHILD', 
         stage_run.run_time, '9999-01-01-12.00.00.000000'
  from twg.Comp prodComp, twg.Comp divComp, cod.stage_measurement stage,
       twg.CompAttr osAttr, twg.CompAttr prodAttr, twg.compAttr custAttr, 
       cod.stage_run_date stage_run
  where divComp.CompTyp_CD = 'COD_DIVISION'
    and prodComp.CompTyp_CD = 'COD_PRODUCT'
    and prodComp.Comp_Nm = stage.prod_name 
    and divComp.Comp_Nm = stage.division_name
    and osAttr.attrTyp_cd = 'COD_OS'
    and prodAttr.attrTyp_cd = 'COD_PRODUCT'
    and custAttr.attrTyp_cd = 'COD_CUSTOMER'
    and osAttr.compAttr_Val = stage.os_version
    and prodAttr.CompAttr_Val = stage.prod_name
    and custAttr.CompAttr_Val = stage.cust_name
    and osAttr.Comp_ID = divComp.Comp_id
    and prodAttr.Comp_ID = divComp.Comp_ID
    and custAttr.Comp_ID = divComp.Comp_ID 
    and not exists
        (select 1 from cod.stage_temp temp
            where temp.prod_name = stage.prod_name
              and temp.division_name = stage.division_name
              and temp.os_version = stage.os_version
              and temp.cust_name = stage.cust_name);
              
-- Add new IP_HOST components
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP;

--#EXECUTE_AT_TARGET
create table cod.stage_temp (
    Hostname        varchar(60) not null
    );

--#EXECUTE_AT_TARGET
create index cod.stage_tempI on cod.stage_temp
(
    Hostname
);


--#EXECUTE_AT_TARGET
insert into cod.stage_temp
    select distinct comp.Comp_Nm
    from twg.Comp comp
    where (comp.CompTyp_cd = 'IP_HOST' or
           comp.CompTyp_cd = 'COD_HOST');
    
--#EXECUTE_AT_TARGET
insert into cod.stage_ip_host
    select distinct hostname from cod.stage_new_comp stage
    where not exists (select 1 from cod.stage_temp temp 
                      where temp.hostname = stage.hostname);

--#EXECUTE_AT_TARGET
insert into twg.Comp (Comp_id, CompTyp_Cd, Centr_Cd, Cust_ID,
                      Comp_Nm, Comp_Strt_DtTm, Comp_End_DtTm)
       select 1, CASE when locate('.',stagehost.name)>0 then 'IP_HOST'
                 else 'COD_HOST' end, 
              'CDW', 1, stagehost.name, 
              CURRENT TIMESTAMP - CURRENT TIMEZONE, '9999-01-01-12.00.00.000000'
              from cod.stage_ip_host stagehost; 

--Add new COD_OS components. There will be a component instance for each
-- actual instance of an OS installed on an IP_HOST (each combination of
-- host/os)
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
DROP TABLE COD.STAGE_TEMP;

--#EXECUTE_AT_TARGET
create table cod.stage_temp (
    hostname            varchar(60)     not null,
    os_version          varchar(60)     not null
);

--#EXECUTE_AT_TARGET
create index cod.stage_tempI on cod.stage_temp
(   
    hostname,
    os_version
);

--#EXECUTE_AT_TARGET
insert into cod.stage_temp
  select distinct hostComp.Comp_Nm, osComp.Comp_Nm
    from twg.Comp osComp, twg.CompReln reln,
         twg.Comp hostComp
    where reln.RelnTyp_Cd = 'LCONT'
      and (hostComp.CompTyp_Cd = 'IP_HOST'
           or hostComp.CompTyp_Cd = 'COD_HOST')
      and osComp.CompTyp_Cd = 'COD_OS'
      and reln.Comp_Target_ID = osComp.Comp_ID
      and reln.Comp_Source_ID = hostComp.Comp_ID;

--#EXECUTE_AT_TARGET
insert into cod.stage_host_os
    select distinct 1, stage.hostname, stage.os_version
    from cod.stage_new_comp stage
    where not exists (select 1 from cod.stage_temp temp
                      where temp.hostname = stage.hostname
                        and temp.os_version = stage.os_version);

-- Insert os components into component table
--#EXECUTE_AT_TARGET
insert into twg.Comp (comp_id, CompTyp_Cd, Centr_Cd, Cust_ID,
                      Comp_Corr_id, Comp_Nm, Comp_Strt_DtTm, Comp_End_DtTm)
       select 1, 'COD_OS', 'CDW', 1, stage.corr_id, 
              stage.os_name, CURRENT TIMESTAMP - CURRENT TIMEZONE, 
              '9999-01-01-12.00.00.000000'
       from cod.stage_host_os stage;

--Insert reln rows between newly added OSs and their host system.
--#EXECUTE_AT_TARGET
insert into twg.CompReln (CompReln_ID, Comp_Source_ID, Comp_Target_ID,
                          RelnTyp_Cd, CompReln_Strt_Dttm, CompReln_End_Dttm) 
  select 1, hostComp.Comp_ID, osComp.Comp_ID, 'LCONT', 
         current timestamp - current timezone, '9999-01-01-12.00.00.000000'
  from twg.Comp osComp, twg.Comp hostComp, cod.stage_host_os stage 
  where (hostComp.CompTyp_Cd = 'IP_HOST' or hostComp.CompTYP_Cd = 'COD_HOST')
    and hostComp.Comp_Nm = stage.host_name
    and osComp.CompTyp_Cd = 'COD_OS'
    and osComp.Comp_Corr_ID = stage.corr_id;

--Insert relationships between Host-OS components and the products installed 
-- on (INSTON) them.
--#EXECUTE_AT_TARGET
insert into twg.CompReln (CompReln_ID, Comp_Source_ID, Comp_Target_ID,
                          RelnTyp_Cd, CompReln_Strt_Dttm, CompReln_End_Dttm) 
  select distinct 1, prodComp.Comp_ID, osComp.Comp_ID, 'INSTON', 
         stage_run.run_time, '9999-01-01-12.00.00.000000'
  from twg.Comp osComp, twg.Comp hostComp, twg.Comp prodComp, 
       twg.Comp divComp, cod.stage_new_comp stage, twg.CompReln lcontReln,
       twg.CompReln pchildReln, twg.CompAttr custAttr, 
       twg.compAttr prodAttr, twg.CompAttr osAttr, cod.stage_run_date stage_run
  where (hostComp.CompTyp_Cd = 'IP_HOST' or hostComp.CompTYP_Cd = 'COD_HOST')
    and hostComp.Comp_Nm = stage.hostname
    and osComp.CompTyp_cd = 'COD_OS'
    and osComp.Comp_Nm = stage.os_version
    and lcontReln.Comp_Source_ID = hostComp.Comp_ID
    and lcontReln.Comp_Target_ID = osComp.Comp_ID
    and lcontReln.RelnTyp_cd = 'LCONT'
    and prodComp.CompTyp_Cd = 'COD_PRODUCT'
    and prodComp.Comp_Nm = stage.prod_name
    and divComp.CompTyp_Cd = 'COD_DIVISION'
    and divComp.Comp_Nm = stage.division_name
    and pchildReln.RelnTyp_Cd = 'PCHILD'
    and pchildReln.Comp_Source_ID = prodComp.Comp_ID
    and pchildReln.Comp_Target_ID = divComp.Comp_ID
    and custAttr.AttrTyp_Cd = 'COD_CUSTOMER'
    and custAttr.CompAttr_Val = stage.cust_name
    and custAttr.Comp_id = divComp.Comp_id
    and prodAttr.AttrTyp_cd = 'COD_PRODUCT'
    and prodAttr.CompAttr_Val = stage.prod_name
    and prodAttr.Comp_id = divComp.Comp_id
    and osAttr.AttrTyp_Cd = 'COD_OS'
    and osAttr.CompAttr_Val = stage.os_version
    and osAttr.Comp_id = divComp.Comp_ID;

--Drop temporary triggers
--
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
drop trigger cod.corr_id_host_os;
--#IGNORE_ERRORS
--#EXECUTE_AT_TARGET
drop trigger COD.CORR_ID_MEASURE;
