-- *****************************************************************************
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-D33
-- * (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- *		
-- * US Government Users Restricted Rights - Use, duplication or
-- * disclosure restricted by GSA ADP Schedule Contract with
-- * IBM Corp.
-- *
-- *****************************************************************************

--
-- Here we perform an incremental extract using integer numbers to control the
-- extraction window; the source tables are the TWG schema tables located in the
-- CDW database.  The target tables are dimension and fact tables located in the
-- Datamart database.
--

--
-- Set the high value for the data to be extracted
--
UPDATE TWG.Extract_Control SET
    extctl_to_intseq = (SELECT MAX(Metric_ID) FROM COD.STAGE_DATABASE_MET)
WHERE
    extctl_source = 'COD.STAGE_DATABASE_MET' AND
    extctl_target = 'COD.D_DIVISION_METRIC'  AND
    (SELECT COUNT(*) FROM COD.STAGE_DATABASE_MET) > 0
;

-- Fill the Metric Dimension Table with the static data retrieved at initialization
-- time and stored into the COD.D_DIVISION_METRIC table.

--#INSERT_INTO_TARGET
INSERT INTO COD.D_DIVISION_METRIC
SELECT
    d.metric_id     AS metric_id,
    d.met_category  AS met_category,
    d.met_desc      AS met_desc,
    d.met_name      AS met_name,
    d.met_units     AS met_units,
    d.min_exists    AS min_exists,
    d.max_exists    AS max_exists,
    d.avg_exists    AS avg_exists,
    d.total_exists  AS total_exists,
    d.msrc_nm       AS msrc_nm
FROM
    COD.STAGE_DATABASE_MET d,
    TWG.Extract_Control ec
WHERE
    ec.extctl_source = 'COD.STAGE_DATABASE_MET' AND
    ec.extctl_target = 'COD.D_DIVISION_METRIC' AND
    d.metric_id > ec.extctl_from_intseq AND
    d.metric_id <= ec.extctl_to_intseq
;

-- Confirm the insertion of data by inserting a row into the TWG.Extract_Log table.

INSERT INTO TWG.Extract_Log
SELECT
    ec.extctl_source        AS extlog_source,
    ec.extctl_target        AS extlog_target,
    CURRENT TIMESTAMP - 
    CURRENT timezone        AS extlog_done_dttm,
    ec.extctl_from_rawseq   AS extlog_from_rawseq,
    ec.extctl_to_rawseq     AS extlog_to_rawseq,
    ec.extctl_from_intseq   AS extlog_from_intseq,
    ec.extctl_to_intseq     AS extlog_to_intseq,
    ec.extctl_from_dttm     AS extlog_from_dttm,
    ec.extctl_to_dttm       AS extlog_to_dttm
FROM
    TWG.EXTRACT_CONTROL ec
WHERE
    ec.extctl_source = 'COD.STAGE_DATABASE_MET' AND
    ec.extctl_target = 'COD.D_DIVISION_METRIC'
;

--
-- Fill the Division Dimension Table with data from the Data Warehouse
--

UPDATE TWG.Extract_Control SET
    extctl_to_intseq = (SELECT MAX(COMP_ID) FROM TWG.Comp)
WHERE
    extctl_source = 'TWG.COMP' AND
    extctl_target = 'COD.D_DIVISION' AND
    (SELECT COUNT(*) FROM TWG.Comp) > 0
;


--
-- Insert into the Division Dimension Table all products not IBM TIVOLI LICENSE MANAGER AGENT
--

--#INSERT_INTO_TARGET
INSERT INTO COD.D_DIVISION
SELECT
    c.Comp_ID               AS division_id,
    c.Comp_Nm               AS division_name,
    prod.CompAttr_Val       AS product_name,
    os.CompAttr_Val         AS os,
    cust.CompAttr_Val       AS customer
FROM
    TWG.Comp c, TWG.CompAttr prod, TWG.CompAttr cust, 
    TWG.CompAttr os, TWG.Extract_Control ec
WHERE
    ec.extctl_source = 'TWG.COMP' AND
    ec.extctl_target = 'COD.D_DIVISION' AND
    c.Comp_ID > ec.extctl_from_intseq AND
    c.Comp_ID <= ec.extctl_to_intseq AND
    c.Comp_ID = prod.Comp_ID AND
    prod.AttrTyp_Cd = 'COD_PRODUCT' AND
    c.Comp_ID = cust.Comp_ID AND
    cust.AttrTyp_Cd = 'COD_CUSTOMER' AND
    c.Comp_ID = os.Comp_ID AND
    os.AttrTyp_Cd = 'COD_OS' AND
    ucase(prod.CompAttr_Val) not like 'IBM TIVOLI LICENSE MANAGER AGENT%'
;

--
-- Insert into the Division Dimension Table only IBM TIVOLI LICENSE MANAGER AGENT
--
-- The reports of ITLM Wep show only the GOLD ITLM Agent (release 1.0, 1.1, 1.1.1 and 1.1.2)
-- but not for the agent patched (release like 1.01, 1.1.0.1, 1.1.1.1).
-- Into the transformation of CDW data into MART data the name of ITLM Agent patched will change
-- on respective GOLD ITLM Agent. 
--
-- For the old Gold Agent (like IBM TIVOLI LICENSE MANAGER AGENT 1.0 or 1.1)
-- "product_name" is "prod.CompAttr_Val"
-- For the Agent whit the length name great of 'IBM TIVOLI LICENSE MANAGER AGENT X.X'
-- the following cases are:
--
-- Agent IBM TIVOLI LICENSE MANAGER AGENT 1.1.0.XYZ patch of IBM TIVOLI LICENSE MANAGER AGENT 1.1
-- (a dot after the second number and the third number is 0)
-- the "product_name" is a substring of "prod.CompAttr_Val" like IBM TIVOLI LICENSE MANAGER AGENT 1.1
-- 
-- Agent IBM TIVOLI LICENSE MANAGER AGENT 1.1.X and related patch IBM TIVOLI LICENSE MANAGER AGENT 1.1.X.YZK
-- (a dot after the second number and the third number is not 0)
-- the "product_name" is a substring of "prod.CompAttr_Val" like IBM TIVOLI LICENSE MANAGER AGENT 1.1.X 
--
-- Agent IBM TIVOLI LICENSE MANAGER AGENT 1.01 patch of IBM TIVOLI LICENSE MANAGER AGENT 1.0
-- (after the second number not a dot)
-- the "product_name" is a substring of "prod.CompAttr_Val" like IBM TIVOLI LICENSE MANAGER AGENT 1.0 
--

--#INSERT_INTO_TARGET
INSERT INTO COD.D_DIVISION
SELECT
    c.Comp_ID               AS division_id,
    c.Comp_Nm               AS division_name,
    CASE 
     WHEN LENGTH(prod.CompAttr_Val) <= LENGTH('IBM TIVOLI LICENSE MANAGER AGENT X.X') THEN prod.CompAttr_Val
     ELSE CASE SUBSTR(prod.CompAttr_Val, LENGTH('IBM TIVOLI LICENSE MANAGER AGENT X.X.'), 1)
           WHEN '.' THEN CASE SUBSTR(prod.CompAttr_Val, LENGTH('IBM TIVOLI LICENSE MANAGER AGENT X.X.X'), 1)
                          WHEN '0' THEN SUBSTR(prod.CompAttr_Val, 1, LENGTH('IBM TIVOLI LICENSE MANAGER AGENT X.X'))  
                          ELSE SUBSTR(prod.CompAttr_Val, 1, LENGTH('IBM TIVOLI LICENSE MANAGER AGENT X.X.X'))
                         END
           ELSE SUBSTR(prod.CompAttr_Val, 1, LENGTH('IBM TIVOLI LICENSE MANAGER AGENT X.X'))
          END
    END                     AS product_name,
    os.CompAttr_Val         AS os,
    cust.CompAttr_Val       AS customer
FROM
    TWG.Comp c, TWG.CompAttr prod, TWG.CompAttr cust, 
    TWG.CompAttr os, TWG.Extract_Control ec
WHERE
    ec.extctl_source = 'TWG.COMP' AND
    ec.extctl_target = 'COD.D_DIVISION' AND
    c.Comp_ID > ec.extctl_from_intseq AND
    c.Comp_ID <= ec.extctl_to_intseq AND
    c.Comp_ID = prod.Comp_ID AND
    prod.AttrTyp_Cd = 'COD_PRODUCT' AND
    c.Comp_ID = cust.Comp_ID AND
    cust.AttrTyp_Cd = 'COD_CUSTOMER' AND
    c.Comp_ID = os.Comp_ID AND
    os.AttrTyp_Cd = 'COD_OS' AND
    ucase(prod.CompAttr_Val) like 'IBM TIVOLI LICENSE MANAGER AGENT%'
;    

-- Confirm the insertion of data by inserting a rwo into the TWG.Extract_Log table.

INSERT INTO TWG.Extract_Log
SELECT
    ec.extctl_source        AS extlog_source,
    ec.extctl_target        AS extlog_target,
    CURRENT TIMESTAMP - 
    CURRENT timezone        AS extlog_done_dttm,
    ec.extctl_from_rawseq   AS extlog_from_rawseq,
    ec.extctl_to_rawseq     AS extlog_to_rawseq,
    ec.extctl_from_intseq   AS extlog_from_intseq,
    ec.extctl_to_intseq     AS extlog_to_intseq,
    ec.extctl_from_dttm     AS extlog_from_dttm,
    ec.extctl_to_dttm       AS extlog_to_dttm
FROM
    TWG.EXTRACT_CONTROL ec
WHERE
    ec.extctl_source = 'TWG.COMP' AND
    ec.extctl_target = 'COD.D_DIVISION'
;

--
-- Log the pruning of the daily fact table COD.F_DIVISION_DAY
--

--#EXECUTE_AT_TARGET
INSERT INTO COD.Prune_Mart_log values 
    (CURRENT TIMESTAMP - CURRENT timezone,
    'COD.F_DIVISION_DAY',
    (SELECT PMartC_Duration from COD.Prune_Mart_Control
     WHERE Table_Name = 'COD.F_DIVISION_DAY'),
        (SELECT COUNT(*) FROM COD.F_DIVISION_DAY f
         WHERE (DATE(CURRENT TIMESTAMP - CURRENT timezone) - DATE(f.meas_date)) >
         (SELECT PMartC_Duration FROM COD.Prune_Mart_Control p
          WHERE p.Table_Name = 'COD.F_DIVISION_DAY')
         )
);          

--
-- Pruning of the daily fact table
--

--#EXECUTE_AT_TARGET
DELETE FROM COD.F_DIVISION_DAY f
WHERE
    (DATE(CURRENT TIMESTAMP - CURRENT timezone) - DATE(f.meas_date)) >
    (SELECT PMartC_Duration from COD.Prune_Mart_Control p
     WHERE p.Table_Name = 'COD.F_DIVISION_DAY'
);


--
-- Fill the Fact Table with the data retrieved from TWG.MSMT table
--

UPDATE TWG.Extract_Control SET
    extctl_to_intseq = (SELECT MAX(msmt_id) FROM TWG.Msmt)
WHERE
    extctl_source = 'TWG.MSMT' AND
    extctl_target = 'COD.F_DIVISION_DAY' AND
    (SELECT COUNT(*) FROM TWG.Msmt) > 0
;

--#INSERT_INTO_TARGET
INSERT INTO COD.F_DIVISION_DAY
SELECT
    m.MsmtTyp_ID                                AS metric_id,
    m.Comp_ID                                   AS division_id,
    TIMESTAMP(m.Msmt_Strt_Dt, '00.00.00')       AS meas_date,
    DECIMAL(m.Msmt_Min_Val, 10, 2)              AS min_value,
    DECIMAL(m.Msmt_Max_Val, 10, 2)              AS max_value
FROM
    TWG.Msmt m, TWG.Comp c, TWG.MsmtTyp t, TWG.Extract_Control ec
WHERE
    m.Comp_ID = c.Comp_ID AND
    m.MsmtTyp_ID = t.MsmtTyp_ID AND
    t.MSrc_cd = 'COD' AND
    ec.extctl_source = 'TWG.MSMT' AND
    ec.extctl_target = 'COD.F_DIVISION_DAY' AND
    m.Msmt_ID > ec.extctl_from_intseq AND
    m.Msmt_ID <= ec.extctl_to_intseq
GROUP BY
    m.Msmt_Strt_Dt, m.Comp_ID, m.MsmtTyp_Id, m.Msmt_Min_Val, m.Msmt_Max_Val
;

-- Confirm the insertion of data by inserting a row into the TWG.Extract_Log table.

INSERT INTO TWG.Extract_Log
SELECT
    ec.extctl_source        AS extlog_source,
    ec.extctl_target        AS extlog_target,
    CURRENT TIMESTAMP - 
    CURRENT timezone        AS extlog_done_dttm,
    ec.extctl_from_rawseq   AS extlog_from_rawseq,
    ec.extctl_to_rawseq     AS extlog_to_rawseq,
    ec.extctl_from_intseq   AS extlog_from_intseq,
    ec.extctl_to_intseq     AS extlog_to_intseq,
    ec.extctl_from_dttm     AS extlog_from_dttm,
    ec.extctl_to_dttm       AS extlog_to_dttm
FROM
    TWG.EXTRACT_CONTROL ec
WHERE
    ec.extctl_source = 'TWG.MSMT' AND
    ec.extctl_target = 'COD.F_DIVISION_DAY'
;