#!/bin/sh
######################################################################
#
# Tivoli Enterprise Data Warehouse, v 1.1.0
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#
#######################################################################
#
# File Information: src/rpi/schema/app_after.sh, twh_rpi, twh_1.1_ode
#
# Revision: 1.4
#
# Date: 7/17/02 15:01:58
#
# Desc:  This is the template for WPack application after scripts.
#        It is intended to be used by WPack's that have additional
#        activities needed during the WPack installation, but that
#        there is no default logic in the WPack installer to handle.
#        There is a hook to execute a Core after script that is
#        required to allow dynamic patching of the WPack install
#        process.  Do not remove this hook.
#
#       NOTE - This after script only runs on the Control Server
#              and not on other distributed hosts.
#
# Command Line Args:
#       1 = the path and name of the TWH Application config file
#       2 = Application working directory or install base directory
#               $TWH_TOPDIR/apps/ava_code_for_app/vapp_version
#               (look at install tree)
#
# Required Config File:
#       twh_app_install.cfg
#
# Installation Logs (Default names):
#       A log will be generated for this scirpt called
#       <ava>_after.sh.log
#
#
# Required Env Vars:
#       TWH_TOPDIR
#
# Return Codes (Only!):
#       0 = Success
#       255 = Failure
#
# Interpreter Required:
#       NT/2000 = BASH
#       Unix    = SH (Bourne Shell)
#
#
# Message Types:
#       (I) - Informational
#       (W) - Warning
#       (F) - Fatal
#
# Install Steps Required Before this Script Executes:
#        * TWH Core tools components must be successfully installed
#
# Note - Caution needs to be taken for NT paths that have a space.
#        This is a limitation that requires double quoting around all
#        variables that contain a path.  Testing should be done to
#        ensure this script runs with a space in any of the paths 
#        that are used.  For example "c:/program files".
#
######################################################################
#

echo "                ** After Script Log **"
echo ""
echo "****************************************"
echo "* Application After Script Start point *"
echo "****************************************"

# Set parms
echo "      Parm 1 = $1"; TWH_CONFIG_FILE="$1"; export TWH_CONFIG_FILE
echo "      Parm 2 = $2"; APP_WORK_DIR="$2"; export APP_WORK_DIR
echo ""

# TWH_TOPDIR needs to always be in the environment
if [ -z "$TWH_TOPDIR" ]; then
  echo "      (F) Could not locate Env var TWH_TOPDIR"
  exit 255
fi

# Execute startup driver for afterscript
. "$TWH_TOPDIR"/install/bin/twh_core_startup_after.sh
exec_startup_driver;

#####################################################
######## Add application code below here ONLY #######
#####################################################


set_cod_program_vars()
{

# name of the temporary sql file
tmp_mart_sql_file="cod_mart_tmp_sql.sql"

# string of the temporary sql file
tmp_sql_COD_MART="ALTER TABLE COD.D_DIVISION 
             ALTER COLUMN product_name SET DATA TYPE VARCHAR(254) 
             ALTER COLUMN os SET DATA TYPE VARCHAR(254) 
             ALTER COLUMN customer SET DATA TYPE VARCHAR(254)";
             

TRUE=1
FALSE=0

return 0;
}


update_cod_data_mart()
{
echo "==> Alter COD.D_DIVISION fields and cleanup records for the COD Data Mart..."

  # Test a connection
  echo "      (I) Testing the DB connection"
  err_txt=`$DB2EXEC_LOG connect to "$TWH_DM_DB" user "$COPT_MART_DB2USER" using "$COPT_MART_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_DM_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  # Update it...
  echo "      (I) Update the COD Data Mart"
  echo "$tmp_sql_COD_MART" > "$TMPDIR/$tmp_mart_sql_file"
  gr_build_exec_script "$TWH_DM_DB" "$TMPDIR" "$tmp_mart_sql_file" $COPT_MART_TYPE $CONN_TYPE_CONNECT;
  gr_exec_sql_script "$TWH_DM_DB" $TRUE;
  rm -f "$TMPDIR/$tmp_mart_sql_file"

  echo "      (I) Done updating the COD Data Mart"

return 0;
}


################
# Main section #
################
echo "==> Processing main WPack routines"
echo ""

# Call your routines
set_cod_program_vars;
update_cod_data_mart;


echo ""
echo "      (I) Done processing main WPack routines"
#####################################
## Stop adding application code #####
#####################################
exec_core_after;		# DO NOT MOVE OR REMOVE!


echo ""
echo "==> Completed executing application after script"
exit 0
